package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.streams.StreamObject;

import java.io.File;
import java.io.IOException;
import java.util.List;

/**
 * This interface describes a file converter
 */
public interface Converter {

    /**
     * Gets all valid input formats for the converter
     *
     * @return <p>A list of valid input formats</p>
     */
    String[] getValidFormats();

    /**
     * Converts the given file
     *
     * @param file <p>The file to convert.</p>
     * @throws IOException <p>If the file cannot be converted.</p>
     */
    void convert(File file) throws IOException;

    /**
     * Generates a command for a ProcessBuilder.
     *
     * @param executable <p>The executable file for ffmpeg.</p>
     * @param file       <p>The input file.</p>
     * @param streams    <p>A list of ffprobe streams.</p>
     * @param outFile    <p>The output file.</p>
     * @return <p>A list of commands</p>
     */
    String[] generateConversionCommand(String executable, File file, List<StreamObject> streams, String outFile);

}
