package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.streams.StreamObject;
import net.knarcraft.ffmpegconverter.streams.SubtitleStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * A converter to embed adjacent subtitles into mp4 files
 */
public class SubtitleEmbed extends AbstractConverter {

    /**
     * Initializes variables used by the abstract converter
     *
     * @param ffprobePath <p>Path/command to ffprobe.</p>
     * @param ffmpegPath  <p>Path/command to ffmpeg.</p>
     */
    public SubtitleEmbed(String ffprobePath, String ffmpegPath) {
        super(null);
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
    }

    @Override
    public String[] getValidFormats() {
        return new String[]{"mp4"};
    }

    @Override
    public String[] generateConversionCommand(String executable, File file, List<StreamObject> streams, String outFile) {
        List<String> command = FFMpegHelper.getFFMpegGeneralFileCommand(executable, file.getName());

        List<SubtitleStream> subtitleStreams = filterStreamsByType(streams, SubtitleStream.class);
        List<SubtitleStream> externalSubtitles = new ArrayList<>();
        for (SubtitleStream subtitleStream : subtitleStreams) {
            if (!subtitleStream.isInternalSubtitle()) {
                externalSubtitles.add(subtitleStream);
            }
        }

        if (externalSubtitles.isEmpty()) {
            System.err.println("No external subtitles found for " + file.getName());
        }

        for (SubtitleStream subtitleStream : externalSubtitles) {
            command.add("-i");
            command.add(subtitleStream.getFile());
        }

        command.add("-c");
        command.add("copy");
        command.add("-c:s");
        command.add("mov_text");

        int i = 0;
        for (SubtitleStream subtitleStream : subtitleStreams) {
            if (subtitleStream.getLanguage() != null) {
                command.add("-metadata:s:s:" + i);
                command.add("language=" + subtitleStream.getLanguage());
            }
            i++;
        }

        if (this.debug) {
            FFMpegHelper.addDebugArguments(command, 50, 120);
        }

        command.add(outFile);
        return command.toArray(new String[0]);
    }

}
